require_relative 'init'
require 'cgi'

module BraceMarkup
  VERSION = '0.0.6'
  DEFAULT_FILE_EXTENSION = '.html'

  # Todo: Rename the main namespace to BraceMarkup
  include ::Bytewise

  #
  # Renders a given raw string with a given context.
  #
  def self.render(input, context: nil, vars: {}, filename: nil)
    # Parse the given string
    ast = self.parse(input)
    context = Context.new(vars, filename: filename) unless context.is_a? Context


    # Calls the renderer
    Renderer.render(ast, context)
  end

  #
  # Renders a file
  #
  def self.render_file(file, context = {})
    self.render(get_file_contents(file), context, filename: file)
  end

  #
  # Parses a file and returns the AST objects.
  #
  def self.parse_file(file)
    self.parse(get_file_contents(file))
  end

  #
  # Parses a raw input
  #
  def self.parse(input)
    Parser.new.parse(input)
  end

  private

  def self.get_file_contents(file)
    # Auto - append the filename if there is none

    if File.extname(file).empty?
      file += DEFAULT_FILE_EXTENSION
    end

    if File.file?(file) && File.readable?(file)
      File.open(file, "rb").read
    else
      raise Exception.new "Could not open file '#{file}'"
    end
  end
end
