module Bytewise
  module Ast
    #
    # This is a basic tag
    # @abstract
    #
    class Tag < Node
      # @deprecated
      ALLOWED_TAGS_NAMES = [:+, :<, :>, :'@', :%, :'#']
      TAGS = {}

      attr_reader :name, :body, :arguments, :sections

      #
      # Registers a tag with a given name
      #
      def self.map(type)
        type = type.to_sym if type.is_a? String

        TAGS[type] = self.name
      end

      #
      # Creates a tag from the tag identifier (+, >, <, #, etc.)
      #
      def self.create(type, ** args)
        if TAGS.key? type
          klass = Object.const_get(TAGS[type])
          klass.new(** args)
        else
          raise Exception.new("Unknown tag type '#{type.to_s}'")
        end
      end

      def initialize(name:, arguments: {}, body: nil, sections: {})
        @name = name
        @body = body
        @sections = sections
        @arguments = arguments
      end

      def render_arguments(*args)
        rendered_arguments = {}

        @arguments.each_pair do |key, value|
          if value.is_a? Node
            rendered_arguments[key] = value.render(*args)
          else
            rendered_arguments[key] = value
          end
        end

        rendered_arguments
      end
    end
  end
end