module Bytewise
  class Context
    attr_reader :vars, :klass
    attr_accessor :filename

    ##
    # +command+:: klass { Object } The current controller context
    #
    def initialize(klass, variables: {}, filename: nil)
      @vars = variables
      @klass = klass
      @filename = filename
    end

    #
    # First tries to resolve the reference in the variables, then
    # in the current class (controller) context
    #
    def resolve_reference(name)
      ref = Context.resolve_reference(name, @vars)

      if ref.nil?
        ref = Context.resolve_reference(name, @klass)
      end

      ref
    end

    ##
    # Resolves a reference key in a specified target
    #
    # @param name { String|Array }
    # @param target { Object }
    #
    def self.resolve_reference(name, target)
      parts = name.is_a?(Array) ? name : name.split('.')
      first = parts.shift
      value = nil

      if target.class.method_defined? :'[]'
        # Is numeric
        if first =~ /^[0-9]+$/
          value = target[first.to_i]
        end

        unless target.is_a? Array
          # First try it with the Symbol
          value = target[first.to_sym] if value.nil?
          # ... then with the String
          value = target[first] if value.nil?
        end
      end

      if value.nil?
        begin
          value = target.send(first)
        rescue NoMethodError
          begin
            value = target.instance_variable_get("@#{first}")
          rescue NameError
            value = nil
          end
        end
      end

      parts.length < 1 ? value : self.resolve_reference(parts, value)
    end
  end
end