module Bytewise::Filters
  class Filter
    FILTERS = {}

    #
    # This method is the actual filter method
    # the return will be inserted to the current context.
    #
    # @param input     mixed
    # @param reference ::Bytewise::Ast::Reference
    # @param **args    "Support for future params"
    #
    def apply(input:, reference:, ** args)
      input
    end

    #
    # Maps a filter to a given name
    # Usage:
    #
    # class ToLowerFilter < ::Bytewise::Filters::Filter
    #   map 'to_lower'
    #
    #   def apply(input:, **args)
    #      # Your code goes here.
    #   end
    # end
    #
    def self.map(name)
      name = name.to_sym if name.is_a? String
      FILTERS[name] = self.name
    end

    #
    # Applies a filter with a given name and passes the arguments to #apply
    #
    def self.apply(name, ** args)
      name = name.to_sym if name.is_a? String

      if FILTERS.key? name
        klass = Object.const_get(FILTERS[name])
        filter = klass.new

        filter.apply(** args)
      else
        raise Exception.new("Filter '#{name}' not found.")
      end
    end
  end
end