module Bytewise::Helpers
  class Helper
    HELPERS = {}

    def initialize(tag:, context:, ** args)
      @tag = tag
      @context = context
    end

    #
    # The return value of this method will be written to
    # the chunk
    #
    def render(** args)

    end

    def render_children
      if @tag.body.children?
        @tag.body.render @context
      end
    end

    def render_section(name)
      @tag.sections.each do |s|
        if s.name == name
          return s.render(@context)
        end
      end

      ''
    end

    def get_sections(name)
      result = []

      @tag.sections.each do |s|
        if s.name == name
          result << s
        end
      end

      result
    end

    #
    # Maps a helper with a name
    # Usage:
    #
    # class MathModHelper < ::Bytewise::Helpers::Helper
    #   map :'math.mod'
    #
    #   def render(a:, b:)
    #      a % b
    #   end
    # end
    #
    def self.map(name)
      name = name.to_sym if name.is_a? String
      HELPERS[name] = self.name
    end

    #
    # Applies a filter with a given name and passes the arguments to #apply
    #
    def self.get_klass(name)
      name = name.to_sym if name.is_a? String

      if HELPERS.key? name
        Object.const_get(HELPERS[name])
      else
        raise Exception.new("Helper '#{name}' not found.")
      end
    end
  end
end